@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/exim_monitor/em_globals.c,v 1.16 2009/10/15 15:38:01 tom Exp $ */
+
 /*************************************************
 *                Exim Monitor                    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2007 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -42,10 +44,15 @@ uschar  actioned_message[24];
 uschar *action_required;
 uschar *alternate_config = NULL;
 
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+int     bmi_run                = 0;
+uschar *bmi_verdicts           = NULL;
+#endif
+
 int     body_max = 20000;
 
 uschar *exim_path              = US BIN_DIRECTORY "/exim"
-			"\0<---------------Space to patch exim_path->";
+                        "\0<---------------Space to patch exim_path->";
 
 int     eximon_initialized = FALSE;
 
@@ -110,25 +117,44 @@ that fires up the monitor fishes the value out by using -bP anyway. */
 #define SPOOL_DIRECTORY ""
 #endif
 
-
-uschar *acl_var[ACL_C_MAX+ACL_M_MAX];
-
+tree_node *acl_var_c           = NULL;
+tree_node *acl_var_m           = NULL;
+uschar *active_hostname        = NULL;
+BOOL    allow_unqualified_recipient = FALSE;
+BOOL    allow_unqualified_sender = FALSE;
 uschar *authenticated_id       = NULL;
 uschar *authenticated_sender   = NULL;
 
 uschar *big_buffer             = NULL;
 int     big_buffer_size        = BIG_BUFFER_SIZE;
 int     body_linecount         = 0;
+int     body_zerocount         = 0;
 
 BOOL    deliver_firsttime      = FALSE;
 BOOL    deliver_freeze         = FALSE;
 int     deliver_frozen_at      = 0;
 BOOL    deliver_manual_thaw    = FALSE;
+
+#ifndef DISABLE_DKIM
+uschar *dkim_cur_signer          = NULL;
+uschar *dkim_signers             = NULL;
+uschar *dkim_signing_domain      = NULL;
+uschar *dkim_signing_selector    = NULL;
+uschar *dkim_verify_signers      = US"$dkim_signers";
+BOOL    dkim_collect_input       = FALSE;
+BOOL    dkim_disable_verify      = FALSE;
+#endif
+
 BOOL    dont_deliver           = FALSE;
 
+#ifdef WITH_CONTENT_SCAN
+int     fake_response          = OK;
+#endif
+
 header_line *header_last       = NULL;
 header_line *header_list       = NULL;
 
+BOOL    host_lookup_deferred   = FALSE;
 BOOL    host_lookup_failed     = FALSE;
 uschar *interface_address      = NULL;
 int     interface_port         = 0;
@@ -136,6 +162,15 @@ int     interface_port         = 0;
 BOOL    local_error_message    = FALSE;
 uschar *local_scan_data        = NULL;
 BOOL    log_timezone           = FALSE;
+
+#ifdef WITH_CONTENT_SCAN
+uschar *spam_bar               = NULL;
+uschar *spam_report            = NULL;
+uschar *spam_score             = NULL;
+uschar *spam_score_int         = NULL;
+#endif
+
+int     max_received_linelength= 0;
 int     message_age            = 0;
 uschar *message_id;
 uschar *message_id_external;
@@ -149,6 +184,8 @@ gid_t   originator_gid;
 uschar *originator_login;
 uid_t   originator_uid;
 
+uschar *primary_hostname       = NULL;
+
 int     received_count         = 0;
 uschar *received_protocol      = NULL;
 int     received_time          = 0;
@@ -167,6 +204,7 @@ int     sender_host_port       = 0;
 uschar *sender_ident           = NULL;
 BOOL    sender_local           = FALSE;
 BOOL    sender_set_untrusted   = FALSE;
+uschar *smtp_active_hostname   = NULL;
 
 BOOL    split_spool_directory  = FALSE;
 uschar *spool_directory        = US SPOOL_DIRECTORY;
