@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/drtables.c,v 1.10 2009/06/10 07:34:04 tom Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2007 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -87,6 +89,14 @@ be NULL for methods that don't need them. */
 #include "lookups/pgsql.h"
 #endif
 
+#ifdef EXPERIMENTAL_SPF
+#include "lookups/spf.h"
+#endif
+
+#ifdef LOOKUP_SQLITE
+#include "lookups/sqlite.h"
+#endif
+
 #ifdef LOOKUP_TESTDB
 #include "lookups/testdb.h"
 #endif
@@ -111,14 +121,14 @@ lookup_info lookup_list[] = {
 /* cdb lookup in single file */
 
   {
-  US"cdb",			 /* lookup name */
-  lookup_absfile,		 /* uses absolute file name */
+  US"cdb",                       /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
 #ifdef LOOKUP_CDB
-  cdb_open,			 /* open function */
-  cdb_check,			 /* check function */
-  cdb_find,			 /* find function */
-  cdb_close,			 /* close function */
-  NULL,				 /* no tidy function */
+  cdb_open,                      /* open function */
+  cdb_check,                     /* check function */
+  cdb_find,                      /* find function */
+  cdb_close,                     /* close function */
+  NULL,                          /* no tidy function */
   NULL                           /* no quoting function */
 #else
   NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
@@ -212,6 +222,24 @@ of the key strings. */
 #endif
   },
 
+/* Linear search of single file with ip-addresses and networks; shares many
+functions with lsearch. */
+
+  {
+  US"iplsearch",                 /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
+#ifdef LOOKUP_LSEARCH
+  lsearch_open,                  /* open function */
+  lsearch_check,                 /* check function */
+  iplsearch_find,                /* find function */
+  lsearch_close,                 /* close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
 /* LDAP lookup, allowing data from only one entry to be returned */
 
   {
@@ -417,6 +445,40 @@ Shares many functions with lsearch. */
 #endif
   },
 
+/* SPF lookup */
+
+  {
+  US"spf",                       /* lookup name */
+  0,                             /* not absfile, not query style */
+#ifdef EXPERIMENTAL_SPF
+  spf_open,                      /* open function */
+  NULL,                          /* no check function */
+  spf_find,                      /* find function */
+  spf_close,                     /* close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
+/* sqlite lookup */
+
+  {
+  US"sqlite",                    /* lookup name */
+  lookup_absfilequery,           /* query-style lookup, starts with file name */
+#ifdef LOOKUP_SQLITE
+  sqlite_open,                   /* open function */
+  NULL,                          /* no check function */
+  sqlite_find,                   /* find function */
+  sqlite_close,                  /* close function */
+  NULL,                          /* no tidy function */
+  sqlite_quote                   /* quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
 /* Testdb lookup is for testing Exim, not useful for normal running.
 For that reason, we omit the entry entirely when not building it into
 the binary, so that attempts to use it give "unknown lookup type" instead
@@ -485,6 +547,14 @@ set to NULL for those that are not compiled into the binary. */
 #include "auths/cram_md5.h"
 #endif
 
+#ifdef AUTH_CYRUS_SASL
+#include "auths/cyrus_sasl.h"
+#endif
+
+#ifdef AUTH_DOVECOT
+#include "auths/dovecot.h"
+#endif
+
 #ifdef AUTH_PLAINTEXT
 #include "auths/plaintext.h"
 #endif
@@ -499,7 +569,7 @@ auth_info auths_available[] = {
 
 #ifdef AUTH_CRAM_MD5
   {
-  US"cram_md5",				     /* lookup name */
+  US"cram_md5",                              /* lookup name */
   auth_cram_md5_options,
   &auth_cram_md5_options_count,
   &auth_cram_md5_option_defaults,
@@ -510,9 +580,35 @@ auth_info auths_available[] = {
   },
 #endif
 
+#ifdef AUTH_CYRUS_SASL
+  {
+  US"cyrus_sasl",           /* lookup name */
+  auth_cyrus_sasl_options,
+  &auth_cyrus_sasl_options_count,
+  &auth_cyrus_sasl_option_defaults,
+  sizeof(auth_cyrus_sasl_options_block),
+  auth_cyrus_sasl_init,                      /* init function */
+  auth_cyrus_sasl_server,                    /* server function */
+  NULL                                       /* client function */
+  },
+#endif
+
+#ifdef AUTH_DOVECOT
+  {
+  US"dovecot",                                /* lookup name */
+  auth_dovecot_options,
+  &auth_dovecot_options_count,
+  &auth_dovecot_option_defaults,
+  sizeof(auth_dovecot_options_block),
+  auth_dovecot_init,                          /* init function */
+  auth_dovecot_server,                        /* server function */
+  NULL                                        /* client function */
+  },
+#endif
+
 #ifdef AUTH_PLAINTEXT
   {
-  US"plaintext",			     /* lookup name */
+  US"plaintext",                             /* lookup name */
   auth_plaintext_options,
   &auth_plaintext_options_count,
   &auth_plaintext_option_defaults,
@@ -525,7 +621,7 @@ auth_info auths_available[] = {
 
 #ifdef AUTH_SPA
   {
-  US"spa",				     /* lookup name */
+  US"spa",                                   /* lookup name */
   auth_spa_options,
   &auth_spa_options_count,
   &auth_spa_option_defaults,
@@ -690,7 +786,6 @@ router_info routers_available[] = {
   ri_notransport
   },
 #endif
-
 { US"", NULL, NULL, NULL, 0, NULL, NULL, NULL, 0 }
 };
 
